<?php

/**
 * generated from https://codekoalas.com/blog/working-plugins-drupal-8
 */

namespace Drupal\entity_ref_recursive\Plugin\Field\FieldFormatter;

use Drupal\Component\Utility\Html;
use Drupal\Core\Field\FieldItemInterface;
use Drupal\Core\Field\FieldItemListInterface;
use Drupal\Core\Field\FormatterBase;
use Drupal\Core\Form\FormStateInterface;

/**
 * Plugin implementation of the 'entity_ref_recursive' formatter.
 *
 * @FieldFormatter(
 *   id = "entity_ref_recursive",
 *   label = @Translation("Recursive labels"),
 *   field_types = {
 *     "entity_reference"
 *   }
 * )
 */
class EntityRefRecursiveFormatter extends FormatterBase {

  /**
   * {@inheritdoc}
   */
  public static function defaultSettings() {
    return [
      // Implement default settings.
    ] + parent::defaultSettings();
  }

  /**
   * {@inheritdoc}
   */
  public function settingsForm(array $form, FormStateInterface $form_state) {
    return [
      // Implement settings form.
    ] + parent::settingsForm($form, $form_state);
  }

  /**
   * {@inheritdoc}
   */
  public function settingsSummary() {
    $summary = [];
    // Implement settings summary.

    return $summary;
  }

  /**
   * {@inheritdoc}
   */
  public function viewElements(FieldItemListInterface $items, $langcode) {
    $root = [];
    $entity = [];
    $children = [];
    
    $this->processParents(0, $items, $root, $children, $entity);
    
    $result = $this->renderItems($root, $children, $entity);
    
    return $result ? [ $result ] : []; 
  }


  /**
   * Fill arrays with entity tree information
   *
   * @param int my_id
   *   entity ID
   * @param FieldItemListInterface $items
   *   Field item list
   * @param array &$root 
   *   List of root IDs
   * @param array &$children
   *   List of lists of children IDs, key is parent ID
   * @param array &$entity
   *   List of entity objects, key is entity ID
   */
  protected function processParents ($my_id, $items, &$root, &$children, &$entity) {
    if ($items->isEmpty()) {
      $root[] = $my_id;
    } else {
      $field_name = $items->getFieldDefinition()->getName();
    
      foreach ($items as $item) {
        $id=$item->getValue()['target_id'];
        $children[$id][] = $my_id;
        if (! isset($entity[$id])) {
          $entity[$id] = \Drupal::entityTypeManager()->getStorage('node')->load($id);
          $entity_items = $entity[$id]->get($field_name);
          $this->processParents ($id, $entity_items, $root, $children, $entity);
        }
      }
    }
  }

  /**
   * Generate the render array based on array tree information
   *
   * @param array &$root
   *   List of root IDs
   * @param array &$children
   *   List of lists of children IDs, key is parent ID
   * @param array &$entity
   *   List of entity objects, key is entity ID
   *   
   * @return array
   *   renderable array
   */
  protected function renderItems (&$root, &$children, &$entity) {
    $result = [];
    
    // need at least one non-zero entry
    if ($root && $root[0]) {
      $result['#theme'] = 'item_list';
      $result['#attributes']['class'][] = 'entity-ref-recursive-list';
      $result['#attached']['library'][] = 'entity_ref_recursive/default';
      $items = [];
      foreach ($root as $id) {
        if ($id) {
          $item = [];
          $item['#markup'] = $entity[$id]->toLink()->toString();
          $item['#wrapper_attributes']['class'][] = 'entity-ref-recursive-list__item';
          $my_list = $this->renderItems($children[$id], $children, $entity);
          if ($my_list) {
            $item[] = $my_list;
          }
          $items[] = $item;
        }
      }
      $result['#items'] = $items;
    }
    return $result;
  }
  
}
